__all__ = ("s3", "fastly", "ConveyorError")

import logging

from pkg_resources import DistributionNotFound, get_distribution

from . import fastly, s3
from .exceptions import ConveyorError

# Allow applications to control the logging handler
logging.getLogger(__name__).addHandler(logging.NullHandler())

try:
    __version__ = get_distribution("ltd-conveyor").version
except DistributionNotFound:
    # Package is not installed
    __version__ = "unknown"



"""Tests for `ltdconveyor.fastly`."""

import uuid

import pytest
import responses

from ltdconveyor.fastly import FastlyError, purge_key


@responses.activate
def test_purge_key() -> None:
    service_id = "SU1Z0isxPaozGVKXdv0eY"
    api_key = "d3cafb4dde4dbeef"
    surrogate_key = uuid.uuid4().hex

    url = "https://api.fastly.com/service/{0}/purge/{1}".format(
        service_id, surrogate_key
    )

    # Mock the API call and response
    responses.add(responses.POST, url, status=200)

    purge_key(surrogate_key, service_id, api_key)
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == url
    assert responses.calls[0].request.headers["Fastly-Key"] == api_key
    assert responses.calls[0].request.headers["Accept"] == "application/json"


@responses.activate
def test_purge_key_fail() -> None:
    service_id = "SU1Z0isxPaozGVKXdv0eY"
    api_key = "d3cafb4dde4dbeef"
    surrogate_key = uuid.uuid4().hex

    url = "https://api.fastly.com/service/{0}/purge/{1}".format(
        service_id, surrogate_key
    )

    # Mock the API call and response
    responses.add(responses.POST, url, status=404)

    with pytest.raises(FastlyError):
        purge_key(surrogate_key, service_id, api_key)
    assert len(responses.calls) == 1
    assert responses.calls[0].request.url == url
    assert responses.calls[0].request.headers["Fastly-Key"] == api_key
    assert responses.calls[0].request.headers["Accept"] == "application/json"



"""Delete an S3 directory."""

import logging
from typing import Any, Dict, Optional

import boto3

from ltdconveyor.s3.exceptions import S3Error

__all__ = ["delete_dir"]


def delete_dir(
    bucket_name: str,
    root_path: str,
    aws_access_key_id: Optional[str] = None,
    aws_secret_access_key: Optional[str] = None,
    aws_profile: Optional[str] = None,
) -> None:
    """Delete all objects in the S3 bucket named ``bucket_name`` that are
    found in the ``root_path`` directory.

    Parameters
    ----------
    bucket_name : `str`
        Name of an S3 bucket.
    root_path : `str`
        Directory in the S3 bucket that will be deleted.
    aws_access_key_id : `str`
        The access key for your AWS account. Also set
        ``aws_secret_access_key``.
    aws_secret_access_key : `str`
        The secret key for your AWS account.
    aws_profile : `str`, optional
        Name of AWS profile in :file:`~/.aws/credentials`. Use this instead
        of ``aws_access_key_id`` and ``aws_secret_access_key`` for file-based
        credentials.

    Raises
    ------
    ltdconveyor.s3.S3Error
        Thrown by any unexpected faults from the S3 API.
    """
    logger = logging.getLogger(__name__)

    session = boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    s3 = session.resource("s3")
    client = s3.meta.client

    # Normalize directory path for searching patch prefixes of objects
    if not root_path.endswith("/"):
        root_path.rstrip("/")

    paginator = client.get_paginator("list_objects_v2")
    pages = paginator.paginate(Bucket=bucket_name, Prefix=root_path)

    keys: Dict[str, Any] = dict(Objects=[])
    for item in pages.search("Contents"):
        try:
            keys["Objects"].append({"Key": item["Key"]})
        except TypeError:  # item is None; nothing to delete
            continue
        # Delete immediately when 1000 objects are listed
        # the delete_objects method can only take a maximum of 1000 keys
        if len(keys["Objects"]) >= 1000:
            try:
                client.delete_objects(Bucket=bucket_name, Delete=keys)
            except Exception:
                message = "Error deleting objects from %r" % root_path
                logger.exception(message)
                raise S3Error(message)
            keys = dict(Objects=[])

    # Delete remaining keys
    if len(keys["Objects"]) > 0:
        try:
            client.delete_objects(Bucket=bucket_name, Delete=keys)
        except Exception:
            message = "Error deleting objects from %r" % root_path
            logger.exception(message)
            raise S3Error(message)



"""Tests for the ``ltdconveyor.s3.presignedpost`` module."""

from __future__ import annotations

import cgi
import sys
from io import BytesIO
from pathlib import Path, PosixPath
from typing import TYPE_CHECKING, Dict

import pytest
import responses

from ltdconveyor.exceptions import ConveyorError
from ltdconveyor.s3.exceptions import S3Error
from ltdconveyor.s3.presignedpost import (
    format_relative_dirname,
    prescan_directory,
    upload_dir,
    upload_directory_objects,
    upload_file,
)

if TYPE_CHECKING:
    from unittest.mock import Mock


@pytest.mark.parametrize(
    "directory,basedir,expected",
    [
        ("/home/project/sub/dir", "/home/project/", "sub/dir/"),
        ("/home/project/sub/dir", "/home/project", "sub/dir/"),
        ("/home/project/sub/dir/", "/home/project", "sub/dir/"),
        ("/home/project/", "/home/project/", "/"),
    ],
)
def test_relative_dirname(directory: str, basedir: str, expected: str) -> None:
    _directory = PosixPath(directory)
    _basedir = PosixPath(basedir)
    result = format_relative_dirname(_directory, _basedir)
    assert result == expected


def test_prescan_directory() -> None:
    """Test the prescan_directory function using a test directory in
    ``tests/data/test-site/``
    """
    base_dir = Path(__file__).parent / "data/test-site"
    assert base_dir.is_dir()

    dirs = prescan_directory(base_dir)
    assert "/" in dirs
    assert "a/" in dirs
    assert "a/aa/" in dirs
    assert "b/" in dirs
    assert len(dirs) == 4


def test_upload_dir(mocker: Mock) -> None:
    mock_upload_file = mocker.patch("ltdconveyor.s3.presignedpost.upload_file")

    post_urls = {
        "/": {
            "url": "https://example.com",
            "fields": {"key": "/prefix/{$filename}"},
        },
        "a/": {
            "url": "https://example.com",
            "fields": {"key": "/prefix/a/{$filename}"},
        },
        "a/aa/": {
            "url": "https://example.com",
            "fields": {"key": "/prefix/a/aa/{$filename}"},
        },
        "b/": {
            "url": "https://example.com",
            "fields": {"key": "/prefix/b/{$filename}"},
        },
    }
    base_dir = Path(__file__).parent / "data/test-site"

    upload_dir(post_urls=post_urls, base_dir=base_dir)

    assert mock_upload_file.call_count == 4


def test_upload_dir_bad_posturls(mocker: Mock) -> None:
    """Test upload_dir when post_urls do not include the directory."""
    mocker.patch("ltdconveyor.s3.presignedpost.upload_file")

    post_urls: Dict[str, str] = {}
    base_dir = Path(__file__).parent / "data/test-site"

    with pytest.raises(ConveyorError):
        upload_dir(post_urls=post_urls, base_dir=base_dir)


@responses.activate
def test_upload_file() -> None:
    local_path = Path(__file__).parent / "data/test-site/index.html"
    post_url = "https://example.com"
    post_fields = {"key": "bucket/base/${filename}"}

    responses.add(responses.POST, "https://example.com", status=204)

    upload_file(
        local_path=local_path, post_url=post_url, post_fields=post_fields
    )

    call = responses.calls[0]
    mimetype, options = cgi.parse_header(call.request.headers["Content-Type"])
    assert mimetype == "multipart/form-data"
    pdict = {
        "boundary": bytes(options["boundary"].encode("ascii")),
        "CONTENT-LENGTH": call.request.headers["Content-Length"].encode(
            "ascii"
        ),
    }
    assert isinstance(responses.calls[0].request.body, bytes)
    data = BytesIO(bytes(responses.calls[0].request.body))
    parsed_body = cgi.parse_multipart(data, pdict)

    if sys.version_info[:3] >= (3, 7, 0):
        assert parsed_body["Content-Type"] == ["text/html"]
        assert parsed_body["key"] == ["bucket/base/${filename}"]
    else:
        assert parsed_body["Content-Type"] == [b"text/html"]
        assert parsed_body["key"] == [b"bucket/base/${filename}"]


@responses.activate
def test_upload_file_failed() -> None:
    local_path = Path(__file__).parent / "data/test-site/index.html"
    post_url = "https://example.com"
    post_fields = {"key": "bucket/base/${filename}"}

    responses.add(responses.POST, "https://example.com", status=403)

    with pytest.raises(S3Error):
        upload_file(
            local_path=local_path, post_url=post_url, post_fields=post_fields
        )


@responses.activate
def test_upload_directory_objects() -> None:
    post_urls = {
        "/": {"url": "https://example.com", "fields": {"key": "bucket/base"}},
        "a/": {
            "url": "https://example.com",
            "fields": {"key": "bucket/base/a"},
        },
        "a/aa/": {
            "url": "https://example.com",
            "fields": {"key": "bucket/base/a/aa"},
        },
        "b/": {
            "url": "https://example.com",
            "fields": {"key": "bucket/base/b/{$filename}"},
        },
    }
    responses.add(responses.POST, "https://example.com", status=204)

    upload_directory_objects(post_urls=post_urls)

    assert len(responses.calls) == 4



"""Tests for ``ltdconveyor.s3upload.upload_dir``.

These tests required the following environment variables:

``LTD_TEST_AWS_ID``
      AWS access key ID

``LTD_TEST_AWS_SECRET``
      AWS secret access key

``LTD_TEST_BUCKET``
      Name of an S3 bucket that already exists and can be used for testing.

The tests will be skipped if they are not available.

Note that this test will create a random uuid4) directory at the root of
``LTD_TEST_BUCKET``, though the test harness will attempt to delete it.
"""

from __future__ import annotations

import logging
import mimetypes
import os
import shutil
import tempfile
import uuid
from typing import TYPE_CHECKING, Any, Dict, List, Sequence

import boto3
import pytest
import requests

from ltdconveyor.s3 import upload_dir

if TYPE_CHECKING:
    from _pytest.fixtures import FixtureRequest

log = logging.getLogger(__name__)


@pytest.mark.skipif(
    os.getenv("LTD_TEST_AWS_ID") is None
    or os.getenv("LTD_TEST_AWS_SECRET") is None
    or os.getenv("LTD_TEST_BUCKET") is None,
    reason="Set LTD_TEST_AWS_ID, "
    "LTD_TEST_AWS_SECRET and "
    "LTD_TEST_BUCKET",
)
def test_upload_dir(request: FixtureRequest) -> None:
    """Integration test of upload_dir and synchronnization upon file deletes.

    This is a test that operates more as an integrations test than a unit
    test.
    """
    bucket_name = os.getenv("LTD_TEST_BUCKET", "")
    aws_access_key_id = os.getenv("LTD_TEST_AWS_ID", "")
    aws_secret_access_key = os.getenv("LTD_TEST_AWS_SECRET", "")
    session = boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    logging.basicConfig(level=logging.ERROR)

    logging.getLogger("ltdconveyor.s3upload").level = logging.INFO
    logging.getLogger(__name__).level = logging.INFO

    paths = [
        "file1.txt",
        "file2.txt",
        "dir1/file11.txt",
        "dir1/file12.txt",
        "dir1/dir11/file111.txt",
        "dir1/dir11/file112.txt",
        "dir2/file21.txt",
        "dir2/file22.txt",
    ]

    temp_dir = tempfile.mkdtemp()
    temp_bucket_dir = str(uuid.uuid4())

    def cleanup() -> None:
        print("Cleaning up the bucket")
        _clean_bucket(session, bucket_name, temp_bucket_dir)

    request.addfinalizer(cleanup)

    _create_test_files(temp_dir, paths)

    surrogate_key = "test-surrogate-key"
    cache_control_max_age = 3600
    upload_dir(
        bucket_name,
        temp_bucket_dir,
        temp_dir,
        surrogate_key=surrogate_key,
        upload_dir_redirect_objects=True,
        cache_control="max-age={0:d}".format(cache_control_max_age),
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    _test_objects_exist(session, bucket_name, temp_bucket_dir, paths)
    _test_content_types(session, bucket_name, temp_bucket_dir)
    _test_directory_redirects(session, bucket_name, temp_bucket_dir, paths)

    expected_headers = {
        "x-amz-meta-surrogate-key": surrogate_key,
        "Cache-Control": "max-age={0:d}".format(cache_control_max_age),
    }
    _test_headers(session, bucket_name, temp_bucket_dir, expected_headers)

    # Remove some files (on filesystem and in path manifest)
    shutil.rmtree(os.path.join(temp_dir, "dir1"))
    paths.remove("dir1/file11.txt")
    paths.remove("dir1/file12.txt")
    paths.remove("dir1/dir11/file111.txt")
    paths.remove("dir1/dir11/file112.txt")
    os.remove(os.path.join(temp_dir, "file2.txt"))
    paths.remove("file2.txt")

    upload_dir(
        bucket_name,
        temp_bucket_dir,
        temp_dir,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    _test_objects_exist(session, bucket_name, temp_bucket_dir, paths)

    shutil.rmtree(temp_dir)


def _create_test_files(temp_dir: str, file_list: List[str]) -> None:
    for path in file_list:
        _write_file(temp_dir, path)


def _write_file(root_dir: str, rel_path: str) -> None:
    filepath = os.path.join(root_dir, rel_path)
    try:
        os.makedirs(os.path.dirname(filepath))
    except OSError:
        # probably exists; pickup real error on write
        pass

    with open(filepath, "w") as f:
        f.write("Content of {0}".format(os.path.basename(filepath)))


def _test_objects_exist(
    session: boto3.session.Session,
    bucket_name: str,
    bucket_root: str,
    file_list: List[str],
) -> None:
    s3 = session.resource("s3")
    bucket = s3.Bucket(bucket_name)

    # dirnames = _file_list_dirnames(file_list, bucket_root)

    bucket_objects = []
    for obj in bucket.objects.filter(Prefix=bucket_root):
        # skip directory objects
        if not os.path.splitext(obj.key)[-1]:
            continue
        bucket_objects.append(obj.key)

    assert len(bucket_objects) == len(file_list)
    for path in file_list:
        found = False
        for bucket_path in bucket_objects:
            if bucket_path.endswith(path):
                found = True
                continue
        if not found:
            log.error("{0} not found in bucket".format(bucket_path))
            assert False


def _test_headers(
    session: boto3.session.Session,
    bucket_name: str,
    bucket_root: str,
    expected_headers: Dict[str, str],
) -> None:
    """Generically test that header key-value pairs in `expected_headers`
    actually are served by S3.
    """
    s3 = session.resource("s3")
    bucket = s3.Bucket(bucket_name)

    # see http://stackoverflow.com/a/34698521 for making object URLs
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for obj in bucket.objects.filter(Prefix=bucket_root):
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location["LocationConstraint"], bucket_name, obj.key
        )
        r = requests.head(object_url)
        # skip directory redirect objects
        if "x-amz-ltd-redirect" in r.headers:
            continue
        for key, expected_value in expected_headers.items():
            assert r.headers[key] == expected_value


def _test_content_types(
    session: boto3.session.Session, bucket_name: str, bucket_root: str
) -> None:
    """Verify that the expected Content-Type header was set."""
    s3 = session.resource("s3")
    bucket = s3.Bucket(bucket_name)

    # AWS api doesn't give Content-Type header, so we'll test it directly
    # via HTTP
    # see http://stackoverflow.com/a/34698521 for making object URLs
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for obj in bucket.objects.filter(Prefix=bucket_root):
        # skip directory redirect objects
        if "ltd-redirect" in obj.Object().metadata:
            continue
        guess, _ = mimetypes.guess_type(obj.key)
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location["LocationConstraint"], bucket_name, obj.key
        )
        if guess is not None:
            r = requests.head(object_url)
            assert r.headers["content-type"] == guess


def _file_list_dirnames(
    file_list: Sequence[str], bucket_root: str
) -> List[str]:
    dirnames = [bucket_root]
    for path in file_list:
        _d = os.path.dirname(path)
        if len(_d) > 0:
            dirnames.append(os.path.join(bucket_root, _d))
    dirnames = list(set(dirnames))
    return dirnames


def _test_directory_redirects(
    session: boto3.session.Session,
    bucket_name: str,
    bucket_root: str,
    file_list: Sequence[str],
) -> None:
    """Verify that the directory redirect objects exist."""
    # Make a list of all directories, including the root directory
    dirnames = _file_list_dirnames(file_list, bucket_root)

    # see http://stackoverflow.com/a/34698521 for making object URLs
    s3 = session.resource("s3")
    # bucket = s3.Bucket(bucket_name)
    bucket_location = s3.meta.client.get_bucket_location(Bucket=bucket_name)

    for dirname in dirnames:
        # Try to request the object
        obj = s3.Object(bucket_name, dirname)
        object_url = "https://s3-{0}.amazonaws.com/{1}/{2}".format(
            bucket_location["LocationConstraint"], bucket_name, obj.key
        )
        r = requests.head(object_url)
        assert r.headers["x-amz-meta-dir-redirect"] == "true"


def _clean_bucket(
    session: boto3.session.Session, bucket_name: str, root_path: str
) -> None:
    s3 = session.resource("s3")
    bucket = s3.Bucket(bucket_name)

    # Normalize directory path for searching patch prefixes of objects
    if not root_path.endswith("/"):
        root_path += "/"

    key_objects = [
        {"Key": obj.key} for obj in bucket.objects.filter(Prefix=root_path)
    ]
    if len(key_objects) == 0:
        return
    delete_keys: Dict[str, List[Any]] = {"Objects": []}
    delete_keys["Objects"] = key_objects
    # based on http://stackoverflow.com/a/34888103
    s3.meta.client.delete_objects(Bucket=bucket.name, Delete=delete_keys)



"""S3 Upload client using presigned POST URLs generated by the LTD Keeper
server application.

For more information about S3 presigned POST URLs, see
https://boto3.amazonaws.com/v1/documentation/api/latest/guide/s3-presigned-urls.html#generating-a-presigned-url-to-upload-a-file
"""

from __future__ import annotations

import logging
import mimetypes
from copy import deepcopy
from typing import TYPE_CHECKING, Any, Dict, List, Optional

import requests

from ltdconveyor.exceptions import ConveyorError
from ltdconveyor.s3.exceptions import S3Error

if TYPE_CHECKING:
    from pathlib import Path

__all__ = [
    "prescan_directory",
    "upload_dir",
    "upload_file",
    "upload_directory_objects",
]


def prescan_directory(
    base_dir: Path, _current_dir: Optional[Path] = None
) -> List[str]:
    """Make a list of all directories in a site, including the root
    directory.

    This function is used to the LTD Keeper API what presigned POST URLs to
    create for a build (see `ltdconveyor.keeper.register_build`).

    Parmameters
    -----------
    base_dir : `pathlib.Path`
        The (local) root directory of a web site.

    Returns
    -------
    dirnames : `list`
        A list of directory names, relative to the root directory. The root
        directory is represented by ``"/"``. All directory names end with
        ``"/"``.
    """
    dirs = []
    if _current_dir is None:
        _current_dir = base_dir
    dirs.append(format_relative_dirname(_current_dir, base_dir))
    for path in _current_dir.iterdir():
        if path.is_dir():
            dirs.extend(prescan_directory(base_dir, _current_dir=path))
    return dirs


def format_relative_dirname(directory: Path, base_directory: Path) -> str:
    """Formats a relative directory path in a way that's compatible with the
    presigned POST URLs.

    Parameters
    ----------
    directory : `pathlib.Path`
        The directory to compute a relative path/name for.
    base_directory : `pathlib.Path`
        The base directory.

    Returns
    -------
    name : `str`
        The relative directory name.

        Examples:

        - ``"base/`` relative to ``"/base/"`` is ``"/"``.
        - ``"base/a/`` relative to ``"/base/"`` is ``"a/"``.
        - ``"base/a/b`` relative to ``"/base/"`` is ``"a/b/"``.
    """
    name = str(directory.relative_to(base_directory))
    if name == ".":
        return "/"
    elif not name.endswith("/"):
        return name + "/"
    else:
        return name


def upload_dir(
    *, post_urls: Dict[str, Any], base_dir: Path, _current_dir: Path = None
) -> None:
    """Upload a local directory of files to S3 for an LSST the Docs build.

    Parameters
    ----------
    post_urls : `dict`
        This dictionary is obtained from the ``"post_prefix_urls"`` field of
        the `ltdconveyor.keeper.register_build` function. It contains
        presigned post POST URLs and fields for each directory in the site
        being uploaded (see the ``dirnames`` parameter to
        `~ltdconveyor.keeper.register_build`).
    base_dir : `pathlib.Path`
        Base directory of the site.

    See also
    --------
    upload_directory_objects
    """
    logger = logging.getLogger(__name__)

    if _current_dir is None:
        _current_dir = base_dir

    relative_dir = format_relative_dirname(_current_dir, base_dir)
    try:
        post_url = post_urls[relative_dir]
    except KeyError:
        logger.exception(
            "A presigned POST URL is not available for the " "%s directory",
            relative_dir,
        )
        raise ConveyorError

    for path in _current_dir.iterdir():
        if path.is_file():
            upload_file(
                local_path=path,
                post_url=post_url["url"],
                post_fields=post_url["fields"],
            )
        elif path.is_dir():
            upload_dir(
                post_urls=post_urls, base_dir=base_dir, _current_dir=path
            )


def upload_file(
    *, local_path: Path, post_url: str, post_fields: Dict[str, Any]
) -> None:
    """Upload a file using a presigned POST URL to S3.

    This function should primarily be used by `upload_dir`.

    Parameters
    ----------
    local_path : `pathlib.Path`
        Path to the file being uploaded.
    post_url : `str`
        URL to upload to.
    post_fields : `dict`
        Dictionary of fields for the POST. Generally these fields are created
        by the LST Keeper API server when it generates the presigned URLs.

    Raises
    ------
    ltdconveyor.s3.exceptions.S3Error
        Raised if the upload fails.
    """
    logger = logging.getLogger(__name__)

    filename = local_path.name
    # don't want to globally change post_fields
    post_fields = deepcopy(post_fields)

    # Detect the Content-Type. This is a required field for the post URL.
    content_type, content_encoding = mimetypes.guess_type(
        filename, strict=False
    )
    if content_type is not None:
        post_fields["Content-Type"] = content_type
    else:
        post_fields["Content-Type"] = "application/octet-stream"

    with open(local_path, "rb") as f:
        files = {"file": (filename, f)}
        http_response = requests.post(post_url, data=post_fields, files=files)
    if http_response.status_code == 204:
        logger.debug(
            "Uploaded %s using presigned POST URL fields %s",
            local_path,
            post_fields,
        )
    else:
        logger.error(
            "Error uploading %s (code %i) using presigned POST URL "
            "fields %s",
            local_path,
            http_response.status_code,
            post_fields,
        )
        raise S3Error


def upload_directory_objects(*, post_urls: Dict[str, Any]) -> None:
    """Upload directory redirect objects for an LSST the Docs product build.

    Parameters
    ----------
    post_urls : `dict`
        This dictionary is obtained from the ``"post_dir_urls"`` field of
        the `ltdconveyor.keeper.register_build` function. It contains
        presigned post POST URLs and fields for each directory in the site
        being uploaded (see the ``dirnames`` parameter to
        `~ltdconveyor.keeper.register_build`).

    See also
    --------
    upload_dir

    Notes
    -----
    Directory redirect objects in S3 are named after directories, don't have
    a trailing slash in their key name, and don't have any content. What
    they *do* have is a ``x-amz-meta-dir-redirect`` header key. The LSST the
    Docs Fastly configuration looks for this header, and when detected,
    redirects the request to the associated ``*/index.html`` object.
    """
    logger = logging.getLogger(__name__)
    for dirname, post_url in post_urls.items():
        url = post_url["url"]
        fields = post_url["fields"]
        files = {"file": ("", "")}
        http_response = requests.post(url, data=fields, files=files)
        if http_response.status_code == 204:
            logger.debug("Uploaded directory object for %s", dirname)
        else:
            logger.error(
                "Error uploading directory object for %s (code %i) using "
                "presigned POST URL fields %s",
                dirname,
                http_response.status_code,
                fields,
            )
            raise S3Error
